!MNH_LIC Copyright 1996-2021 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!     ##########################################################################
      SUBROUTINE ICE_ADJUST (KKA, KKU, KKL, KRR, HFRAC_ICE, HCONDENS, HLAMBDA3,&
                             HBUNAME, OSUBG_COND, OSIGMAS, OCND2, LHGT_QS,     &
                             HSUBG_MF_PDF, PTSTEP, PSIGQSAT,                   &
                             PRHODJ, PEXNREF, PRHODREF, PSIGS, PMFCONV,        &
                             PPABST, PZZ,                                      &
                             PEXN, PCF_MF, PRC_MF, PRI_MF,                     &
                             PICLDFR, PWCLDFR, PSSIO, PSSIU, PIFR,             &
                             PRV, PRC, PRVS, PRCS, PTH, PTHS, PSRCS, PCLDFR,   &
                             PRR, PRI, PRIS, PRS, PRG, TBUDGETS, KBUDGETS,     &
                             PICE_CLD_WGT,                                     &
                             PRH,                                              &
                             POUT_RV, POUT_RC, POUT_RI, POUT_TH,               &
                             PHLC_HRC, PHLC_HCF, PHLI_HRI, PHLI_HCF)
!     #########################################################################
!
!!****  *ICE_ADJUST* -  compute the ajustment of water vapor in mixed-phase 
!!                      clouds
!!
!!    PURPOSE
!!    -------
!!    The purpose of this routine is to compute the fast microphysical sources
!!    through a saturation ajustement procedure in case of mixed-phase clouds.
!!
!!
!!**  METHOD
!!    ------
!!    Langlois, Tellus, 1973 for the cloudless version.
!!    When cloud water is taken into account, refer to book 1 of the
!!    documentation.
!!
!!     
!!
!!    EXTERNAL
!!    --------
!!      None
!!     
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!      Module MODD_CST
!!         XP00               ! Reference pressure
!!         XMD,XMV            ! Molar mass of dry air and molar mass of vapor
!!         XRD,XRV            ! Gaz constant for dry air, gaz constant for vapor
!!         XCPD,XCPV          ! Cpd (dry air), Cpv (vapor)
!!         XCL                ! Cl (liquid)
!!         XCI                ! Ci (ice)
!!         XTT                ! Triple point temperature
!!         XLVTT              ! Vaporization heat constant
!!         XLSTT              ! Sublimation  heat constant
!!         XALPW,XBETAW,XGAMW ! Constants for saturation vapor over liquid
!!                            !  pressure  function 
!!         XALPI,XBETAI,XGAMI ! Constants for saturation vapor over ice
!!                            !  pressure  function 
!!      Module  MODD_CONF 
!!         CCONF
!!      Module MODD_BUDGET:
!!         NBUMOD 
!!         CBUTYPE
!!         LBU_RTH    
!!         LBU_RRV  
!!         LBU_RRC  
!!         LBU_RRI  
!!
!!
!!    REFERENCE
!!    ---------
!!      Book 1 and Book2 of documentation ( routine ICE_ADJUST )
!!      Langlois, Tellus, 1973
!!
!!    AUTHOR
!!    ------
!!      J.-P. Pinty    * Laboratoire d'Aerologie*
!!   
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    06/12/96 
!!      M. Tomasini 27/11/00 Change CND and DEP fct of the T instead of rc and ri
!!                           Avoid the sub- and super-saturation before the ajustment
!!                           Avoid rc>0 if T<T00 before the ajustment
!!      P Bechtold 12/02/02  change subgrid condensation
!!      JP Pinty   29/11/02  add ICE2 and IC4 cases
!!      (P. Jabouille) 27/05/04 safety test for case where esw/i(T)> pabs (~Z>40km)
!!      J.Pergaud and S.Malardel Add EDKF case
!!      S. Riette ice for EDKF
!!      2012-02 Y. Seity,  add possibility to run with reversed vertical levels
!!      J.Escobar : 15/09/2015 : WENO5 & JPHEXT <> 1
!!      2016-07 S. Riette: adjustement is now realized on state variables (PRV, PRC, PRI, PTH)
!!                         whereas tendencies are still applied on S variables.
!!                         This modification allows to call ice_adjust on T variable
!!                         or to call it on S variables
!!      2016-11 S. Riette: all-or-nothing adjustment now uses condensation
!  P. Wautelet 05/2016-04/2018: new data structures and calls for I/O
!!      2018-02 K.I.Ivarsson : More outputs for OCND2 option
!  P. Wautelet    02/2020: use the new data structures and subroutines for budgets
!!      2020-12 U. Andrae : Introduce SPP for HARMONIE-AROME
!!     R. El Khatib 24-Aug-2021 Optimizations
!!
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!              ------------
!
USE PARKIND1, ONLY : JPRB
USE YOMHOOK , ONLY : LHOOK, DR_HOOK
USE MODD_BUDGET,     ONLY: TBUDGETDATA, LBU_ENABLE,  &
                         & LBUDGET_TH, LBUDGET_RV, LBUDGET_RC, LBUDGET_RI,  &
                         & NBUDGET_TH, NBUDGET_RV, NBUDGET_RC, NBUDGET_RI
USE MODD_CST,        ONLY: XLVTT, XLSTT, XCPV, XCL, XCI, XTT, XCPD, XCPV
USE MODD_PARAMETERS, ONLY: JPVEXT
USE MODD_RAIN_ICE_PARAM, ONLY : XCRIAUTC, XCRIAUTI, XACRIAUTI, XBCRIAUTI
!
USE MODE_BUDGET,         ONLY: BUDGET_STORE_INIT, BUDGET_STORE_END
USE MODE_ll,             ONLY: GET_INDICE_ll
!
USE MODI_CONDENSATION
!
IMPLICIT NONE
!
!
!*       0.1   Declarations of dummy arguments :
!
!
INTEGER,                  INTENT(IN)    :: KKA      !near ground array index
INTEGER,                  INTENT(IN)    :: KKU      !uppest atmosphere array index
INTEGER,                  INTENT(IN)    :: KKL      !vert. levels type 1=MNH -1=ARO
INTEGER,                  INTENT(IN)    :: KRR      ! Number of moist variables
CHARACTER(LEN=1),         INTENT(IN)    :: HFRAC_ICE
CHARACTER(LEN=80),        INTENT(IN)    :: HCONDENS
CHARACTER(LEN=4),         INTENT(IN)    :: HLAMBDA3 ! formulation for lambda3 coeff
CHARACTER(LEN=4),         INTENT(IN)    :: HBUNAME  ! Name of the budget
LOGICAL,                  INTENT(IN)    :: OSUBG_COND ! Switch for Subgrid
                                                    ! Condensation
LOGICAL,                  INTENT(IN)    :: OSIGMAS  ! Switch for Sigma_s:
                                                    ! use values computed in CONDENSATION
                                                    ! or that from turbulence scheme
LOGICAL,                  INTENT(IN)    :: OCND2    ! logical switch to separate liquid
                                                    ! and ice
                                                    ! more rigid (DEFAULT value : .FALSE.)
LOGICAL,                  INTENT(IN)   :: LHGT_QS   ! logical switch for height dependent VQSIGSAT
CHARACTER(LEN=80),        INTENT(IN)   :: HSUBG_MF_PDF
REAL,                     INTENT(IN)   :: PTSTEP    ! Double Time step
                                                    ! (single if cold start)
REAL, DIMENSION(:,:),     INTENT(IN)   :: PSIGQSAT  ! coeff applied to qsat variance contribution
!
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)   ::  PRHODJ  ! Dry density * Jacobian
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)   ::  PEXNREF ! Reference Exner function
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)   ::  PRHODREF
!
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)   ::  PSIGS   ! Sigma_s at time t
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)   ::  PMFCONV ! convective mass flux
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)   ::  PPABST  ! Absolute Pressure at t
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)   ::  PZZ     ! height of model layer
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)   ::  PEXN    ! Exner function
!
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)    :: PCF_MF  ! Convective Mass Flux Cloud fraction
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)    :: PRC_MF  ! Convective Mass Flux liquid mixing ratio
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)    :: PRI_MF  ! Convective Mass Flux ice mixing ratio
!
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)    :: PRV     ! Water vapor m.r. to adjust
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)    :: PRC     ! Cloud water m.r. to adjust
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(INOUT) :: PRVS    ! Water vapor m.r. source
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(INOUT) :: PRCS    ! Cloud water m.r. source
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(IN)    :: PTH     ! Theta to adjust
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(INOUT) :: PTHS    ! Theta source
REAL, DIMENSION(:,:,:), CONTIGUOUS,   INTENT(OUT)   :: PSRCS   ! Second-order flux
                                                   ! s'rc'/2Sigma_s2 at time t+1
                                                   ! multiplied by Lambda_3
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(OUT)   :: PCLDFR  ! Cloud fraction
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(OUT)   :: PICLDFR ! ice cloud fraction
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(OUT)   :: PWCLDFR ! water or mixed-phase cloud fraction
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(OUT)   :: PSSIO   ! Super-saturation with respect to ice in the  
                                                              ! supersaturated fraction
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(OUT)   :: PSSIU   ! Sub-saturation with respect to ice in the  
                                                              ! subsaturated fraction 
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(OUT)   :: PIFR    ! Ratio cloud ice moist part to dry part
!
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(INOUT) :: PRIS    ! Cloud ice  m.r. at t+1
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(IN)    :: PRR     ! Rain water m.r. to adjust
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(IN)    :: PRI     ! Cloud ice  m.r. to adjust
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(IN)    :: PRS     ! Aggregate  m.r. to adjust
REAL, DIMENSION(:,:,:), CONTIGUOUS,  INTENT(IN)    :: PRG     ! Graupel    m.r. to adjust
TYPE(TBUDGETDATA), DIMENSION(KBUDGETS), INTENT(INOUT) :: TBUDGETS
INTEGER,                             INTENT(IN)    :: KBUDGETS
REAL, DIMENSION(:,:),   CONTIGUOUS, OPTIONAL, INTENT(IN)   ::  PICE_CLD_WGT
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(IN)   ::  PRH  ! Hail       m.r. to adjust
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(OUT)  ::  POUT_RV ! Adjusted value
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(OUT)  ::  POUT_RC ! Adjusted value
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(OUT)  ::  POUT_RI ! Adjusted value
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(OUT)  ::  POUT_TH ! Adjusted value
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(OUT)  ::  PHLC_HRC
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(OUT)  ::  PHLC_HCF
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(OUT)  ::  PHLI_HRI
REAL, DIMENSION(:,:,:), CONTIGUOUS, OPTIONAL, INTENT(OUT)  ::  PHLI_HCF
!
!*       0.2   Declarations of local variables :
!
!
REAL  :: ZW1,ZW2    ! intermediate fields
REAL, DIMENSION(SIZE(PEXNREF,1),SIZE(PEXNREF,2),SIZE(PEXNREF,3)) &
                         :: ZT,   &  ! adjusted temperature
                   ZRV, ZRC, ZRI, &  ! adjusted state
                            ZCPH, &  ! guess of the CPh for the mixing
                            ZLV,  &  ! guess of the Lv at t+1
                            ZLS      ! guess of the Ls at t+1
REAL :: ZCRIAUT, & ! Autoconversion thresholds
        ZHCF, ZHR
!
INTEGER             :: IIU,IJU,IKU! dimensions of dummy arrays
INTEGER             :: IIB,IJB    ! Horz index values of the first inner mass points
INTEGER             :: IIE,IJE    ! Horz index values of the last inner mass points
INTEGER             :: IKB        ! K index value of the first inner mass point
INTEGER             :: IKE        ! K index value of the last inner mass point
INTEGER             :: JITER,ITERMAX ! iterative loop for first order adjustment
INTEGER             :: JI, JJ, JK
!
REAL, DIMENSION(SIZE(PEXNREF,1),SIZE(PEXNREF,2),SIZE(PEXNREF,3)) :: ZSIGS, ZSRCS
REAL, DIMENSION(SIZE(PEXNREF,1),SIZE(PEXNREF,2)) :: ZSIGQSAT
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
!-------------------------------------------------------------------------------
!
!*       1.     PRELIMINARIES
!               -------------
!
IF (LHOOK) CALL DR_HOOK('ICE_ADJUST',0,ZHOOK_HANDLE)
!
IIU = SIZE(PEXNREF,1)
IJU = SIZE(PEXNREF,2)
IKU = SIZE(PEXNREF,3)
CALL GET_INDICE_ll(IIB,IJB,IIE,IJE,IIU,IJU)
IKB=KKA+JPVEXT*KKL
IKE=KKU-JPVEXT*KKL
!
ITERMAX=1
!
IF(LBUDGET_TH) CALL BUDGET_STORE_INIT(TBUDGETS(NBUDGET_TH), TRIM(HBUNAME), PTHS(:, :, :)*PRHODJ(:, :, :))
IF(LBUDGET_RV) CALL BUDGET_STORE_INIT(TBUDGETS(NBUDGET_RV), TRIM(HBUNAME), PRVS(:, :, :)*PRHODJ(:, :, :))
IF(LBUDGET_RC) CALL BUDGET_STORE_INIT(TBUDGETS(NBUDGET_RC), TRIM(HBUNAME), PRCS(:, :, :)*PRHODJ(:, :, :))
IF(LBUDGET_RI) CALL BUDGET_STORE_INIT(TBUDGETS(NBUDGET_RI), TRIM(HBUNAME), PRIS(:, :, :)*PRHODJ(:, :, :))
!-------------------------------------------------------------------------------
!
!*       2.     COMPUTE QUANTITIES WITH THE GUESS OF THE FUTURE INSTANT
!               -------------------------------------------------------
!
!
!    beginning of the iterative loop (to compute the adjusted state)
!
DO JITER =1,ITERMAX
  !
  !*       2.3    compute the latent heat of vaporization Lv(T*) at t+1
  !                   and the latent heat of sublimation  Ls(T*) at t+1
  !
  DO JK=1,IKU
    DO JJ=1,IJU
      DO JI=1,IIU
        IF (JITER==1) ZT(JI,JJ,JK) = PTH(JI,JJ,JK) * PEXN(JI,JJ,JK)
        ZLV(JI,JJ,JK) = XLVTT + ( XCPV - XCL ) * ( ZT(JI,JJ,JK) -XTT )
        ZLS(JI,JJ,JK) = XLSTT + ( XCPV - XCI ) * ( ZT(JI,JJ,JK) -XTT )
      ENDDO
    ENDDO
  ENDDO
  !
  !*       2.4   Iterate
  !
  IF (JITER==1) THEN
    ! compute with input values
    CALL ITERATION(PRV,PRC,PRI,ZRV,ZRC,ZRI)
  ELSE
    ! compute with updated values
    CALL ITERATION(ZRV,ZRC,ZRI,ZRV,ZRC,ZRI)
  ENDIF
ENDDO         ! end of the iterative loop
!
!*       5.     COMPUTE THE SOURCES AND STORES THE CLOUD FRACTION
!               -------------------------------------------------
!
!
DO JK=1,IKU
  DO JJ=1,IJU
    DO JI=1,IIU
      !
      !*       5.0    compute the variation of mixing ratio
      !
                                                           !         Rc - Rc*
      ZW1 = (ZRC(JI,JJ,JK) - PRC(JI,JJ,JK)) / PTSTEP       ! Pcon = ----------
                                                           !         2 Delta t
      ZW2 = (ZRI(JI,JJ,JK) - PRI(JI,JJ,JK)) / PTSTEP       ! idem ZW1 but for Ri
      !
      !*       5.1    compute the sources
      !
      IF( ZW1 < 0.0 ) THEN
        ZW1 = MAX ( ZW1, -PRCS(JI,JJ,JK) )
      ELSE
        ZW1 = MIN ( ZW1,  PRVS(JI,JJ,JK) )
      ENDIF
      PRVS(JI,JJ,JK) = PRVS(JI,JJ,JK) - ZW1
      PRCS(JI,JJ,JK) = PRCS(JI,JJ,JK) + ZW1
      PTHS(JI,JJ,JK) = PTHS(JI,JJ,JK) +        &
                      ZW1 * ZLV(JI,JJ,JK) / (ZCPH(JI,JJ,JK) * PEXNREF(JI,JJ,JK))
      !
      IF( ZW2 < 0.0 ) THEN
        ZW2 = MAX ( ZW2, -PRIS(JI,JJ,JK) )
      ELSE
        ZW2 = MIN ( ZW2,  PRVS(JI,JJ,JK) )
      ENDIF
      PRVS(JI,JJ,JK) = PRVS(JI,JJ,JK) - ZW2
      PRIS(JI,JJ,JK) = PRIS(JI,JJ,JK) + ZW2
      PTHS(JI,JJ,JK) = PTHS(JI,JJ,JK) +        &
                    ZW2 * ZLS(JI,JJ,JK) / (ZCPH(JI,JJ,JK) * PEXNREF(JI,JJ,JK))
    ENDDO
    !
    !*       5.2    compute the cloud fraction PCLDFR
    !
    IF ( .NOT. OSUBG_COND ) THEN
      DO JI=1,IIU
        IF (PRCS(JI,JJ,JK) + PRIS(JI,JJ,JK) > 1.E-12 / PTSTEP) THEN
          PCLDFR(JI,JJ,JK)  = 1.
        ELSE
          PCLDFR(JI,JJ,JK)  = 0.
        ENDIF
        IF ( SIZE(PSRCS,3) /= 0 ) THEN
          PSRCS(JI,JJ,JK) = PCLDFR(JI,JJ,JK)
        END IF
      ENDDO
    ELSE !OSUBG_COND case
      DO JI=1,IIU
        !We limit PRC_MF+PRI_MF to PRVS*PTSTEP to avoid negative humidity
        ZW1=PRC_MF(JI,JJ,JK)/PTSTEP
        ZW2=PRI_MF(JI,JJ,JK)/PTSTEP
        IF(ZW1+ZW2>PRVS(JI,JJ,JK)) THEN
          ZW1=ZW1*PRVS(JI,JJ,JK)/(ZW1+ZW2)
          ZW2=PRVS(JI,JJ,JK)-ZW1
        ENDIF
        PCLDFR(JI,JJ,JK)=MIN(1.,PCLDFR(JI,JJ,JK)+PCF_MF(JI,JJ,JK))
        PRCS(JI,JJ,JK)=PRCS(JI,JJ,JK)+ZW1
        PRIS(JI,JJ,JK)=PRIS(JI,JJ,JK)+ZW2
        PRVS(JI,JJ,JK)=PRVS(JI,JJ,JK)-(ZW1+ZW2)
        PTHS(JI,JJ,JK) = PTHS(JI,JJ,JK) + &
                      (ZW1 * ZLV(JI,JJ,JK) + ZW2 * ZLS(JI,JJ,JK)) / ZCPH(JI,JJ,JK) / PEXNREF(JI,JJ,JK)
        !
        IF(PRESENT(PHLC_HRC) .AND. PRESENT(PHLC_HCF)) THEN
          ZCRIAUT=XCRIAUTC/PRHODREF(JI,JJ,JK)
          IF(HSUBG_MF_PDF=='NONE')THEN
            IF(ZW1*PTSTEP>PCF_MF(JI,JJ,JK) * ZCRIAUT) THEN
              PHLC_HRC(JI,JJ,JK)=PHLC_HRC(JI,JJ,JK)+ZW1*PTSTEP
              PHLC_HCF(JI,JJ,JK)=MIN(1.,PHLC_HCF(JI,JJ,JK)+PCF_MF(JI,JJ,JK))
            ENDIF
          ELSEIF(HSUBG_MF_PDF=='TRIANGLE')THEN
            !ZHCF is the precipitating part of the *cloud* and not of the grid cell
            IF(ZW1*PTSTEP>PCF_MF(JI,JJ,JK)*ZCRIAUT) THEN
              ZHCF=1.-.5*(ZCRIAUT*PCF_MF(JI,JJ,JK) / MAX(1.E-20, ZW1*PTSTEP))**2
              ZHR=ZW1*PTSTEP-(ZCRIAUT*PCF_MF(JI,JJ,JK))**3 / &
                                          &(3*MAX(1.E-20, ZW1*PTSTEP)**2)
            ELSEIF(2.*ZW1*PTSTEP<=PCF_MF(JI,JJ,JK) * ZCRIAUT) THEN
              ZHCF=0.
              ZHR=0.
            ELSE
              ZHCF=(2.*ZW1*PTSTEP-ZCRIAUT*PCF_MF(JI,JJ,JK))**2 / &
                         &(2.*MAX(1.E-20, ZW1*PTSTEP)**2)
              ZHR=(4.*(ZW1*PTSTEP)**3-3.*ZW1*PTSTEP*(ZCRIAUT*PCF_MF(JI,JJ,JK))**2+&
                          (ZCRIAUT*PCF_MF(JI,JJ,JK))**3) / &
                        &(3*MAX(1.E-20, ZW1*PTSTEP)**2)
            ENDIF
            ZHCF=ZHCF*PCF_MF(JI,JJ,JK) !to retrieve the part of the grid cell
            PHLC_HCF(JI,JJ,JK)=MIN(1.,PHLC_HCF(JI,JJ,JK)+ZHCF) !total part of the grid cell that is precipitating
            PHLC_HRC(JI,JJ,JK)=PHLC_HRC(JI,JJ,JK)+ZHR
          ENDIF
        ENDIF
        IF(PRESENT(PHLI_HRI) .AND. PRESENT(PHLI_HCF)) THEN
          ZCRIAUT=MIN(XCRIAUTI,10**(XACRIAUTI*(ZT(JI,JJ,JK)-XTT)+XBCRIAUTI))
          IF(HSUBG_MF_PDF=='NONE')THEN
            IF(ZW2*PTSTEP>PCF_MF(JI,JJ,JK) * ZCRIAUT) THEN
              PHLI_HRI(JI,JJ,JK)=PHLI_HRI(JI,JJ,JK)+ZW2*PTSTEP
              PHLI_HCF(JI,JJ,JK)=MIN(1.,PHLI_HCF(JI,JJ,JK)+PCF_MF(JI,JJ,JK))
            ENDIF
          ELSEIF(HSUBG_MF_PDF=='TRIANGLE')THEN
            !ZHCF is the precipitating part of the *cloud* and not of the grid cell
            IF(ZW2*PTSTEP>PCF_MF(JI,JJ,JK)*ZCRIAUT) THEN
              ZHCF=1.-.5*(ZCRIAUT*PCF_MF(JI,JJ,JK) / (ZW2*PTSTEP))**2
              ZHR=ZW2*PTSTEP-(ZCRIAUT*PCF_MF(JI,JJ,JK))**3/(3*(ZW2*PTSTEP)**2)
            ELSEIF(2.*ZW2*PTSTEP<=PCF_MF(JI,JJ,JK) * ZCRIAUT) THEN
              ZHCF=0.
              ZHR=0.
            ELSE
              ZHCF=(2.*ZW2*PTSTEP-ZCRIAUT*PCF_MF(JI,JJ,JK))**2 / (2.*(ZW2*PTSTEP)**2)
              ZHR=(4.*(ZW2*PTSTEP)**3-3.*ZW2*PTSTEP*(ZCRIAUT*PCF_MF(JI,JJ,JK))**2+&
                          (ZCRIAUT*PCF_MF(JI,JJ,JK))**3)/(3*(ZW2*PTSTEP)**2)
            ENDIF
            ZHCF=ZHCF*PCF_MF(JI,JJ,JK) !to retrieve the part of the grid cell
            PHLI_HCF(JI,JJ,JK)=MIN(1.,PHLI_HCF(JI,JJ,JK)+ZHCF) !total part of the grid cell that is precipitating
            PHLI_HRI(JI,JJ,JK)=PHLI_HRI(JI,JJ,JK)+ZHR
          ENDIF
        ENDIF
      ENDDO
      !
      IF(PRESENT(POUT_RV) .OR. PRESENT(POUT_RC) .OR. &
        &PRESENT(POUT_RI) .OR. PRESENT(POUT_TH)) THEN
        DO JI=1,IIU
          ZW1=PRC_MF(JI,JJ,JK)
          ZW2=PRI_MF(JI,JJ,JK)
          IF(ZW1+ZW2>ZRV(JI,JJ,JK)) THEN
            ZW1=ZW1*ZRV(JI,JJ,JK)/(ZW1+ZW2)
            ZW2=ZRV(JI,JJ,JK)-ZW1
          ENDIF
          ZRC(JI,JJ,JK)=ZRC(JI,JJ,JK)+ZW1
          ZRI(JI,JJ,JK)=ZRI(JI,JJ,JK)+ZW2
          ZRV(JI,JJ,JK)=ZRV(JI,JJ,JK)-(ZW1+ZW2)
          ZT(JI,JJ,JK) = ZT(JI,JJ,JK) + &
                      (ZW1 * ZLV(JI,JJ,JK) + ZW2 * ZLS(JI,JJ,JK)) / ZCPH(JI,JJ,JK)
        ENDDO
      ENDIF
    ENDIF !OSUBG_COND
  ENDDO
ENDDO
!
IF(PRESENT(POUT_RV)) POUT_RV=ZRV
IF(PRESENT(POUT_RC)) POUT_RC=ZRC
IF(PRESENT(POUT_RI)) POUT_RI=ZRI
IF(PRESENT(POUT_TH)) POUT_TH=ZT / PEXN(:,:,:)
!
!
!*       6.  STORE THE BUDGET TERMS
!            ----------------------
!
IF(LBUDGET_TH) CALL BUDGET_STORE_END(TBUDGETS(NBUDGET_TH), TRIM(HBUNAME), PTHS(:, :, :)*PRHODJ(:, :, :))
IF(LBUDGET_RV) CALL BUDGET_STORE_END(TBUDGETS(NBUDGET_RV), TRIM(HBUNAME), PRVS(:, :, :)*PRHODJ(:, :, :))
IF(LBUDGET_RC) CALL BUDGET_STORE_END(TBUDGETS(NBUDGET_RC), TRIM(HBUNAME), PRCS(:, :, :)*PRHODJ(:, :, :))
IF(LBUDGET_RI) CALL BUDGET_STORE_END(TBUDGETS(NBUDGET_RI), TRIM(HBUNAME), PRIS(:, :, :)*PRHODJ(:, :, :))
!------------------------------------------------------------------------------
!
!
IF (LHOOK) CALL DR_HOOK('ICE_ADJUST',1,ZHOOK_HANDLE)
!
CONTAINS
SUBROUTINE ITERATION(PRV_IN,PRC_IN,PRI_IN,PRV_OUT,PRC_OUT,PRI_OUT)

REAL, DIMENSION(:,:,:), CONTIGUOUS, INTENT(IN) :: PRV_IN ! Water vapor m.r. to adjust in input
REAL, DIMENSION(:,:,:), CONTIGUOUS, INTENT(IN) :: PRC_IN ! Cloud water m.r. to adjust in input
REAL, DIMENSION(:,:,:), CONTIGUOUS, INTENT(IN) :: PRI_IN ! Cloud ice   m.r. to adjust in input
REAL, DIMENSION(:,:,:), CONTIGUOUS, INTENT(OUT) :: PRV_OUT ! Water vapor m.r. to adjust in output
REAL, DIMENSION(:,:,:), CONTIGUOUS, INTENT(OUT) :: PRC_OUT ! Cloud water m.r. to adjust in output
REAL, DIMENSION(:,:,:), CONTIGUOUS, INTENT(OUT) :: PRI_OUT ! Cloud ice   m.r. to adjust in output
!
!*       2.4    compute the specific heat for moist air (Cph) at t+1

SELECT CASE(KRR)
  CASE(7)
    ZCPH(:,:,:) = XCPD + XCPV * PRV_IN(:,:,:)                             &
                       + XCL  * (PRC_IN(:,:,:) + PRR(:,:,:))             &
                       + XCI  * (PRI_IN(:,:,:) + PRS(:,:,:) + PRG(:,:,:) + PRH(:,:,:))
  CASE(6)
    ZCPH(:,:,:) = XCPD + XCPV * PRV_IN(:,:,:)                             &
                       + XCL  * (PRC_IN(:,:,:) + PRR(:,:,:))             &
                       + XCI  * (PRI_IN(:,:,:) + PRS(:,:,:) + PRG(:,:,:))
  CASE(5)
    ZCPH(:,:,:) = XCPD + XCPV * PRV_IN(:,:,:)                             &
                       + XCL  * (PRC_IN(:,:,:) + PRR(:,:,:))             &
                       + XCI  * (PRI_IN(:,:,:) + PRS(:,:,:))
  CASE(3)
    ZCPH(:,:,:) = XCPD + XCPV * PRV_IN(:,:,:)               &
                       + XCL  * (PRC_IN(:,:,:) + PRR(:,:,:))
  CASE(2)
    ZCPH(:,:,:) = XCPD + XCPV * PRV_IN(:,:,:) &
                       + XCL  * PRC_IN(:,:,:)
END SELECT
!
IF ( OSUBG_COND ) THEN
  !
  !*       3.     SUBGRID CONDENSATION SCHEME
  !               ---------------------------
  !
  !   PSRC= s'rci'/Sigma_s^2
  !   ZT is INOUT
  CALL CONDENSATION(IIU, IJU, IKU, IIB, IIE, IJB, IJE, IKB, IKE, KKL,    &
       HFRAC_ICE, HCONDENS, HLAMBDA3,                                    &
       PPABST, PZZ, PRHODREF, ZT, PRV_IN, PRV_OUT, PRC_IN, PRC_OUT, PRI_IN, PRI_OUT, &
       PRR, PRS, PRG, PSIGS, PMFCONV, PCLDFR, &
       PSRCS, .TRUE., OSIGMAS, OCND2, LHGT_QS,                           &
       PICLDFR, PWCLDFR, PSSIO, PSSIU, PIFR, PSIGQSAT,                   &
       PLV=ZLV, PLS=ZLS, PCPH=ZCPH,                                      &
       PHLC_HRC=PHLC_HRC, PHLC_HCF=PHLC_HCF, PHLI_HRI=PHLI_HRI, PHLI_HCF=PHLI_HCF,&
       PICE_CLD_WGT=PICE_CLD_WGT)
ELSE
  !
  !*       4.     ALL OR NOTHING CONDENSATION SCHEME
  !                            FOR MIXED-PHASE CLOUD
  !               -----------------------------------------------
  !
  ZSIGS(:,:,:)=0.
  ZSIGQSAT(:,:)=0.
  !We use ZSRCS because in Méso-NH, PSRCS can be a zero-length array in this case
  !ZT is INOUT
  CALL CONDENSATION(IIU, IJU, IKU, IIB, IIE, IJB, IJE, IKB, IKE, KKL,    &
       HFRAC_ICE, HCONDENS, HLAMBDA3,                                    &
       PPABST, PZZ, PRHODREF, ZT, PRV_IN, PRV_OUT, PRC_IN, PRC_OUT, PRI_IN, PRI_OUT, &
       PRR, PRS, PRG, ZSIGS, PMFCONV, PCLDFR,                            &
       ZSRCS, .TRUE., .TRUE., OCND2, LHGT_QS,                            &
       PICLDFR, PWCLDFR, PSSIO, PSSIU, PIFR, ZSIGQSAT,                   &
       PLV=ZLV, PLS=ZLS, PCPH=ZCPH,                                      &
       PHLC_HRC=PHLC_HRC, PHLC_HCF=PHLC_HCF, PHLI_HRI=PHLI_HRI, PHLI_HCF=PHLI_HCF,&
       PICE_CLD_WGT=PICE_CLD_WGT)
ENDIF

END SUBROUTINE ITERATION

END SUBROUTINE ICE_ADJUST 
